@@ -3,50 +3,48 @@ package e2e
 import (
 	"bytes"
 	"context"
-	"encoding/json"
 	"fmt"
 	"strings"
-	"testing"
 	"time"
 
 	"github.com/ghodss/yaml"
-	"github.com/operator-framework/operator-registry/pkg/api/grpc_health_v1"
-	"github.com/sirupsen/logrus"
+	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/gomega"
 	"github.com/stretchr/testify/require"
 	"google.golang.org/grpc"
+	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions"
 	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	"k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	extScheme "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset/scheme"
 	"k8s.io/apimachinery/pkg/api/equality"
-	"k8s.io/apimachinery/pkg/api/errors"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
+	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	k8sjson "k8s.io/apimachinery/pkg/runtime/serializer/json"
 	"k8s.io/apimachinery/pkg/util/diff"
 	"k8s.io/apimachinery/pkg/util/wait"
+	"k8s.io/apimachinery/pkg/watch"
 	"k8s.io/apiserver/pkg/storage/names"
 	"k8s.io/client-go/dynamic"
 	k8sscheme "k8s.io/client-go/kubernetes/scheme"
-	kscheme "k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/rest"
+	"k8s.io/component-base/featuregate"
 
-	operatorsv1 "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1"
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client"
+	"github.com/operator-framework/api/pkg/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/registry"
-	controllerclient "github.com/operator-framework/operator-lifecycle-manager/pkg/lib/controller-runtime/client"
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/feature"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
-	pmclient "github.com/operator-framework/operator-lifecycle-manager/pkg/package-server/client"
 	pmversioned "github.com/operator-framework/operator-lifecycle-manager/pkg/package-server/client/clientset/versioned"
+	"github.com/operator-framework/operator-lifecycle-manager/test/e2e/ctx"
+	"github.com/operator-framework/operator-registry/pkg/api/grpc_health_v1"
 )
 
-
-
 const (
 	pollInterval = 1 * time.Second
 	pollDuration = 5 * time.Minute
@@ -81,63 +79,43 @@ func newNamespaceCleaner(namespace string) *namespaceCleaner {
 }
 
 // notifyOnFailure checks if a test has failed or cleanup is true before cleaning a namespace
-func (c *namespaceCleaner) NotifyTestComplete(t *testing.T, cleanup bool) {
-	if t.Failed() {
+func (c *namespaceCleaner) NotifyTestComplete(cleanup bool) {
+	if CurrentGinkgoTestDescription().Failed {
 		c.skipCleanupOLM = true
 	}
 
 	if c.skipCleanupOLM || !cleanup {
-		t.Log("skipping cleanup")
+		ctx.Ctx().Logf("skipping cleanup")
 		return
 	}
 
-	cleanupOLM(t, c.namespace)
+	cleanupOLM(c.namespace)
 }
 
 // newKubeClient configures a client to talk to the cluster defined by KUBECONFIG
-func newKubeClient(t *testing.T) operatorclient.ClientInterface {
-	if kubeConfigPath == nil {
-		t.Log("using in-cluster config")
-	}
-	// TODO: impersonate ALM serviceaccount
-	// TODO: thread logger from test
-	return operatorclient.NewClientFromConfig(*kubeConfigPath, logrus.New())
+func newKubeClient() operatorclient.ClientInterface {
+	return ctx.Ctx().KubeClient()
 }
 
-func newCRClient(t *testing.T) versioned.Interface {
-	if kubeConfigPath == nil {
-		t.Log("using in-cluster config")
-	}
-	// TODO: impersonate ALM serviceaccount
-	crclient, err := client.NewClient(*kubeConfigPath)
-	require.NoError(t, err)
-	return crclient
+func newCRClient() versioned.Interface {
+	return ctx.Ctx().OperatorClient()
 }
 
-func newDynamicClient(t *testing.T, config *rest.Config) dynamic.Interface {
-	// TODO: impersonate ALM serviceaccount
-	dynamicClient, err := dynamic.NewForConfig(config)
-	require.NoError(t, err)
-	return dynamicClient
+func newDynamicClient(t GinkgoTInterface, config *rest.Config) dynamic.Interface {
+	return ctx.Ctx().DynamicClient()
 }
 
-func newPMClient(t *testing.T) pmversioned.Interface {
-	if kubeConfigPath == nil {
-		t.Log("using in-cluster config")
-	}
-	// TODO: impersonate ALM serviceaccount
-	pmc, err := pmclient.NewClient(*kubeConfigPath)
-	require.NoError(t, err)
-	return pmc
+func newPMClient() pmversioned.Interface {
+	return ctx.Ctx().PackageClient()
 }
 
 // awaitPods waits for a set of pods to exist in the cluster
-func awaitPods(t *testing.T, c operatorclient.ClientInterface, namespace, selector string, checkPods podsCheckFunc) (*corev1.PodList, error) {
+func awaitPods(t GinkgoTInterface, c operatorclient.ClientInterface, namespace, selector string, checkPods podsCheckFunc) (*corev1.PodList, error) {
 	var fetchedPodList *corev1.PodList
 	var err error
 
 	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
-		fetchedPodList, err = c.KubernetesInterface().CoreV1().Pods(namespace).List(metav1.ListOptions{
+		fetchedPodList, err = c.KubernetesInterface().CoreV1().Pods(namespace).List(context.TODO(), metav1.ListOptions{
 			LabelSelector: selector,
 		})
 
@@ -154,13 +132,13 @@ func awaitPods(t *testing.T, c operatorclient.ClientInterface, namespace, select
 	return fetchedPodList, err
 }
 
-func awaitPodsWithInterval(t *testing.T, c operatorclient.ClientInterface, namespace, selector string, interval time.Duration,
+func awaitPodsWithInterval(t GinkgoTInterface, c operatorclient.ClientInterface, namespace, selector string, interval time.Duration,
 	duration time.Duration, checkPods podsCheckFunc) (*corev1.PodList, error) {
 	var fetchedPodList *corev1.PodList
 	var err error
 
 	err = wait.Poll(interval, duration, func() (bool, error) {
-		fetchedPodList, err = c.KubernetesInterface().CoreV1().Pods(namespace).List(metav1.ListOptions{
+		fetchedPodList, err = c.KubernetesInterface().CoreV1().Pods(namespace).List(context.TODO(), metav1.ListOptions{
 			LabelSelector: selector,
 		})
 
@@ -236,10 +214,10 @@ func podReady(pod *corev1.Pod) bool {
 	return status == corev1.ConditionTrue
 }
 
-func awaitPod(t *testing.T, c operatorclient.ClientInterface, namespace, name string, checkPod podCheckFunc) *corev1.Pod {
+func awaitPod(t GinkgoTInterface, c operatorclient.ClientInterface, namespace, name string, checkPod podCheckFunc) *corev1.Pod {
 	var pod *corev1.Pod
 	err := wait.Poll(pollInterval, pollDuration, func() (bool, error) {
-		p, err := c.KubernetesInterface().CoreV1().Pods(namespace).Get(name, metav1.GetOptions{})
+		p, err := c.KubernetesInterface().CoreV1().Pods(namespace).Get(context.TODO(), name, metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -251,12 +229,12 @@ func awaitPod(t *testing.T, c operatorclient.ClientInterface, namespace, name st
 	return pod
 }
 
-func awaitAnnotations(t *testing.T, query func() (metav1.ObjectMeta, error), expected map[string]string) error {
+func awaitAnnotations(t GinkgoTInterface, query func() (metav1.ObjectMeta, error), expected map[string]string) error {
 	var err error
 	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
 		t.Logf("Waiting for annotations to match %v", expected)
 		obj, err := query()
-		if err != nil && !errors.IsNotFound(err) {
+		if err != nil && !apierrors.IsNotFound(err) {
 			return false, err
 		}
 		t.Logf("current annotations: %v", obj.GetAnnotations())
@@ -279,7 +257,7 @@ func awaitAnnotations(t *testing.T, query func() (metav1.ObjectMeta, error), exp
 }
 
 // compareResources compares resource equality then prints a diff for easier debugging
-func compareResources(t *testing.T, expected, actual interface{}) {
+func compareResources(t GinkgoTInterface, expected, actual interface{}) {
 	if eq := equality.Semantic.DeepEqual(expected, actual); !eq {
 		t.Fatalf("Resource does not match expected value: %s",
 			diff.ObjectDiff(expected, actual))
@@ -292,7 +270,7 @@ func waitForDelete(checkResource checkResourceFunc) error {
 	var err error
 	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
 		err := checkResource()
-		if errors.IsNotFound(err) {
+		if apierrors.IsNotFound(err) {
 			return true, nil
 		}
 		if err != nil {
@@ -322,9 +300,9 @@ func waitForEmptyList(checkList func() (int, error)) error {
 
 func waitForGVR(dynamicClient dynamic.Interface, gvr schema.GroupVersionResource, name, namespace string) error {
 	return wait.Poll(pollInterval, pollDuration, func() (bool, error) {
-		_, err := dynamicClient.Resource(gvr).Namespace(namespace).Get(name, metav1.GetOptions{})
+		_, err := dynamicClient.Resource(gvr).Namespace(namespace).Get(context.TODO(), name, metav1.GetOptions{})
 		if err != nil {
-			if errors.IsNotFound(err) {
+			if apierrors.IsNotFound(err) {
 				return false, nil
 			}
 			return false, err
@@ -373,12 +351,12 @@ func catalogSourceRegistryPodSynced(catalog *v1alpha1.CatalogSource) bool {
 	return false
 }
 
-func fetchCatalogSource(t *testing.T, crc versioned.Interface, name, namespace string, check catalogSourceCheckFunc) (*v1alpha1.CatalogSource, error) {
+func fetchCatalogSourceOnStatus(crc versioned.Interface, name, namespace string, check catalogSourceCheckFunc) (*v1alpha1.CatalogSource, error) {
 	var fetched *v1alpha1.CatalogSource
 	var err error
 
 	err = wait.Poll(pollInterval, pollDuration, func() (bool, error) {
-		fetched, err = crc.OperatorsV1alpha1().CatalogSources(namespace).Get(name, metav1.GetOptions{})
+		fetched, err = crc.OperatorsV1alpha1().CatalogSources(namespace).Get(context.TODO(), name, metav1.GetOptions{})
 		if err != nil || fetched == nil {
 			fmt.Println(err)
 			return false, err
@@ -403,77 +381,87 @@ func createFieldNotEqualSelector(field string, names ...string) string {
 	return builder.String()
 }
 
-func cleanupOLM(t *testing.T, namespace string) {
+func cleanupOLM(namespace string) {
 	var immediate int64 = 0
-	crc := newCRClient(t)
-	c := newKubeClient(t)
+	crc := newCRClient()
+	c := newKubeClient()
 
 	// Cleanup non persistent OLM CRs
-	t.Log("cleaning up any remaining non persistent resources...")
-	deleteOptions := &metav1.DeleteOptions{GracePeriodSeconds: &immediate}
+	ctx.Ctx().Logf("cleaning up any remaining non persistent resources...")
+	deleteOptions := metav1.DeleteOptions{GracePeriodSeconds: &immediate}
 	listOptions := metav1.ListOptions{}
-	require.NoError(t, crc.OperatorsV1alpha1().ClusterServiceVersions(namespace).DeleteCollection(deleteOptions, metav1.ListOptions{FieldSelector: nonPersistentCSVFieldSelector}))
-	require.NoError(t, crc.OperatorsV1alpha1().InstallPlans(namespace).DeleteCollection(deleteOptions, listOptions))
-	require.NoError(t, crc.OperatorsV1alpha1().Subscriptions(namespace).DeleteCollection(deleteOptions, listOptions))
-	require.NoError(t, crc.OperatorsV1alpha1().CatalogSources(namespace).DeleteCollection(deleteOptions, metav1.ListOptions{FieldSelector: nonPersistentCatalogsFieldSelector}))
+
+	err := crc.OperatorsV1alpha1().ClusterServiceVersions(namespace).DeleteCollection(context.TODO(), deleteOptions, metav1.ListOptions{FieldSelector: nonPersistentCSVFieldSelector})
+	Expect(err).NotTo(HaveOccurred())
+
+	err = crc.OperatorsV1alpha1().InstallPlans(namespace).DeleteCollection(context.TODO(), deleteOptions, listOptions)
+	Expect(err).NotTo(HaveOccurred())
+
+	err = crc.OperatorsV1alpha1().Subscriptions(namespace).DeleteCollection(context.TODO(), deleteOptions, listOptions)
+	Expect(err).NotTo(HaveOccurred())
+
+	err = crc.OperatorsV1alpha1().CatalogSources(namespace).DeleteCollection(context.TODO(), deleteOptions, metav1.ListOptions{FieldSelector: nonPersistentCatalogsFieldSelector})
+	Expect(err).NotTo(HaveOccurred())
 
 	// error: the server does not allow this method on the requested resource
 	// Cleanup non persistent configmaps
-	require.NoError(t, c.KubernetesInterface().CoreV1().Pods(namespace).DeleteCollection(deleteOptions, metav1.ListOptions{}))
-
-	var err error
-	err = waitForEmptyList(func() (int, error) {
-		res, err := crc.OperatorsV1alpha1().ClusterServiceVersions(namespace).List(metav1.ListOptions{FieldSelector: nonPersistentCSVFieldSelector})
-		t.Logf("%d %s remaining", len(res.Items), "csvs")
-		return len(res.Items), err
-	})
-	require.NoError(t, err)
+	err = c.KubernetesInterface().CoreV1().Pods(namespace).DeleteCollection(context.TODO(), deleteOptions, metav1.ListOptions{})
+	Expect(err).NotTo(HaveOccurred())
 
 	err = waitForEmptyList(func() (int, error) {
-		res, err := crc.OperatorsV1alpha1().InstallPlans(namespace).List(metav1.ListOptions{})
-		t.Logf("%d %s remaining", len(res.Items), "installplans")
+		res, err := crc.OperatorsV1alpha1().ClusterServiceVersions(namespace).List(context.TODO(), metav1.ListOptions{FieldSelector: nonPersistentCSVFieldSelector})
+		ctx.Ctx().Logf("%d %s remaining", len(res.Items), "csvs")
 		return len(res.Items), err
 	})
-	require.NoError(t, err)
+	Expect(err).NotTo(HaveOccurred())
 
 	err = waitForEmptyList(func() (int, error) {
-		res, err := crc.OperatorsV1alpha1().Subscriptions(namespace).List(metav1.ListOptions{})
-		t.Logf("%d %s remaining", len(res.Items), "subs")
+		res, err := crc.OperatorsV1alpha1().InstallPlans(namespace).List(context.TODO(), metav1.ListOptions{})
+		ctx.Ctx().Logf("%d %s remaining", len(res.Items), "installplans")
 		return len(res.Items), err
 	})
-	require.NoError(t, err)
+	Expect(err).NotTo(HaveOccurred())
 
 	err = waitForEmptyList(func() (int, error) {
-		res, err := crc.OperatorsV1alpha1().CatalogSources(namespace).List(metav1.ListOptions{FieldSelector: nonPersistentCatalogsFieldSelector})
-		t.Logf("%d %s remaining", len(res.Items), "catalogs")
+		res, err := crc.OperatorsV1alpha1().Subscriptions(namespace).List(context.TODO(), metav1.ListOptions{})
+		ctx.Ctx().Logf("%d %s remaining", len(res.Items), "subs")
 		return len(res.Items), err
 	})
-	require.NoError(t, err)
+	Expect(err).NotTo(HaveOccurred())
+
+	err = waitForEmptyList(func() (int, error) {
+		res, err := crc.OperatorsV1alpha1().CatalogSources(namespace).List(context.TODO(), metav1.ListOptions{FieldSelector: nonPersistentCatalogsFieldSelector})
+		ctx.Ctx().Logf("%d %s remaining", len(res.Items), "catalogs")
+		return len(res.Items), err
+	})
+	Expect(err).NotTo(HaveOccurred())
 }
 
-func buildCatalogSourceCleanupFunc(t *testing.T, crc versioned.Interface, namespace string, catalogSource *v1alpha1.CatalogSource) cleanupFunc {
+func buildCatalogSourceCleanupFunc(crc versioned.Interface, namespace string, catalogSource *v1alpha1.CatalogSource) cleanupFunc {
 	return func() {
-		t.Logf("Deleting catalog source %s...", catalogSource.GetName())
-		require.NoError(t, crc.OperatorsV1alpha1().CatalogSources(namespace).Delete(catalogSource.GetName(), &metav1.DeleteOptions{}))
+		ctx.Ctx().Logf("Deleting catalog source %s...", catalogSource.GetName())
+		err := crc.OperatorsV1alpha1().CatalogSources(namespace).Delete(context.TODO(), catalogSource.GetName(), metav1.DeleteOptions{})
+		Expect(err).ToNot(HaveOccurred())
 	}
 }
 
-func buildConfigMapCleanupFunc(t *testing.T, c operatorclient.ClientInterface, namespace string, configMap *corev1.ConfigMap) cleanupFunc {
+func buildConfigMapCleanupFunc(c operatorclient.ClientInterface, namespace string, configMap *corev1.ConfigMap) cleanupFunc {
 	return func() {
-		t.Logf("Deleting config map %s...", configMap.GetName())
-		require.NoError(t, c.KubernetesInterface().CoreV1().ConfigMaps(namespace).Delete(configMap.GetName(), &metav1.DeleteOptions{}))
+		ctx.Ctx().Logf("Deleting config map %s...", configMap.GetName())
+		err := c.KubernetesInterface().CoreV1().ConfigMaps(namespace).Delete(context.TODO(), configMap.GetName(), metav1.DeleteOptions{})
+		Expect(err).ToNot(HaveOccurred())
 	}
 }
 
-func buildServiceAccountCleanupFunc(t *testing.T, c operatorclient.ClientInterface, namespace string, serviceAccount *corev1.ServiceAccount) cleanupFunc {
+func buildServiceAccountCleanupFunc(t GinkgoTInterface, c operatorclient.ClientInterface, namespace string, serviceAccount *corev1.ServiceAccount) cleanupFunc {
 	return func() {
 		t.Logf("Deleting service account %s...", serviceAccount.GetName())
-		require.NoError(t, c.KubernetesInterface().CoreV1().ServiceAccounts(namespace).Delete(serviceAccount.GetName(), &metav1.DeleteOptions{}))
+		require.NoError(t, c.KubernetesInterface().CoreV1().ServiceAccounts(namespace).Delete(context.TODO(), serviceAccount.GetName(), metav1.DeleteOptions{}))
 	}
 }
 
-func createInternalCatalogSource(t *testing.T, c operatorclient.ClientInterface, crc versioned.Interface, name, namespace string, manifests []registry.PackageManifest, crds []apiextensions.CustomResourceDefinition, csvs []v1alpha1.ClusterServiceVersion) (*v1alpha1.CatalogSource, cleanupFunc) {
-	configMap, configMapCleanup := createConfigMapForCatalogData(t, c, name, namespace, manifests, crds, csvs)
+func createInternalCatalogSource(c operatorclient.ClientInterface, crc versioned.Interface, name, namespace string, manifests []registry.PackageManifest, crds []apiextensions.CustomResourceDefinition, csvs []v1alpha1.ClusterServiceVersion) (*v1alpha1.CatalogSource, cleanupFunc) {
+	configMap, configMapCleanup := createConfigMapForCatalogData(c, name, namespace, manifests, crds, csvs)
 
 	// Create an internal CatalogSource custom resource pointing to the ConfigMap
 	catalogSource := &v1alpha1.CatalogSource{
@@ -492,21 +480,101 @@ func createInternalCatalogSource(t *testing.T, c operatorclient.ClientInterface,
 	}
 	catalogSource.SetNamespace(namespace)
 
-	t.Logf("Creating catalog source %s in namespace %s...", name, namespace)
-	catalogSource, err := crc.OperatorsV1alpha1().CatalogSources(namespace).Create(catalogSource)
-	if err != nil && !errors.IsAlreadyExists(err) {
-		require.NoError(t, err)
+	ctx.Ctx().Logf("Creating catalog source %s in namespace %s...", name, namespace)
+	catalogSource, err := crc.OperatorsV1alpha1().CatalogSources(namespace).Create(context.TODO(), catalogSource, metav1.CreateOptions{})
+	if err != nil && !apierrors.IsAlreadyExists(err) {
+		Expect(err).ToNot(HaveOccurred())
 	}
-	t.Logf("Catalog source %s created", name)
+	ctx.Ctx().Logf("Catalog source %s created", name)
 
 	cleanupInternalCatalogSource := func() {
 		configMapCleanup()
-		buildCatalogSourceCleanupFunc(t, crc, namespace, catalogSource)()
+		buildCatalogSourceCleanupFunc(crc, namespace, catalogSource)()
 	}
 	return catalogSource, cleanupInternalCatalogSource
 }
 
-func createConfigMapForCatalogData(t *testing.T, c operatorclient.ClientInterface, name, namespace string, manifests []registry.PackageManifest, crds []apiextensions.CustomResourceDefinition, csvs []v1alpha1.ClusterServiceVersion) (*corev1.ConfigMap, cleanupFunc) {
+func createV1CRDInternalCatalogSource(t GinkgoTInterface, c operatorclient.ClientInterface, crc versioned.Interface, name, namespace string, manifests []registry.PackageManifest, crds []apiextensionsv1.CustomResourceDefinition, csvs []v1alpha1.ClusterServiceVersion) (*v1alpha1.CatalogSource, cleanupFunc) {
+	configMap, configMapCleanup := createV1CRDConfigMapForCatalogData(t, c, name, namespace, manifests, crds, csvs)
+
+	// Create an internal CatalogSource custom resource pointing to the ConfigMap
+	catalogSource := &v1alpha1.CatalogSource{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       v1alpha1.CatalogSourceKind,
+			APIVersion: v1alpha1.CatalogSourceCRDAPIVersion,
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: namespace,
+		},
+		Spec: v1alpha1.CatalogSourceSpec{
+			SourceType: "internal",
+			ConfigMap:  configMap.GetName(),
+		},
+	}
+	catalogSource.SetNamespace(namespace)
+
+	ctx.Ctx().Logf("Creating catalog source %s in namespace %s...", name, namespace)
+	catalogSource, err := crc.OperatorsV1alpha1().CatalogSources(namespace).Create(context.TODO(), catalogSource, metav1.CreateOptions{})
+	if err != nil && !apierrors.IsAlreadyExists(err) {
+		require.NoError(t, err)
+	}
+	ctx.Ctx().Logf("Catalog source %s created", name)
+
+	cleanupInternalCatalogSource := func() {
+		configMapCleanup()
+		buildCatalogSourceCleanupFunc(crc, namespace, catalogSource)()
+	}
+	return catalogSource, cleanupInternalCatalogSource
+}
+
+func createConfigMapForCatalogData(c operatorclient.ClientInterface, name, namespace string, manifests []registry.PackageManifest, crds []apiextensions.CustomResourceDefinition, csvs []v1alpha1.ClusterServiceVersion) (*corev1.ConfigMap, cleanupFunc) {
+	// Create a config map containing the PackageManifests and CSVs
+	configMapName := fmt.Sprintf("%s-configmap", name)
+	catalogConfigMap := &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      configMapName,
+			Namespace: namespace,
+		},
+		Data: map[string]string{},
+	}
+	catalogConfigMap.SetNamespace(namespace)
+
+	// Add raw manifests
+	if manifests != nil {
+		manifestsRaw, err := yaml.Marshal(manifests)
+		Expect(err).ToNot(HaveOccurred())
+		catalogConfigMap.Data[registry.ConfigMapPackageName] = string(manifestsRaw)
+	}
+
+	// Add raw CRDs
+	var crdsRaw []byte
+	if crds != nil {
+		crdStrings := []string{}
+		for _, crd := range crds {
+			crdStrings = append(crdStrings, serializeCRD(crd))
+		}
+		var err error
+		crdsRaw, err = yaml.Marshal(crdStrings)
+		Expect(err).ToNot(HaveOccurred())
+	}
+	catalogConfigMap.Data[registry.ConfigMapCRDName] = strings.Replace(string(crdsRaw), "- |\n  ", "- ", -1)
+
+	// Add raw CSVs
+	if csvs != nil {
+		csvsRaw, err := yaml.Marshal(csvs)
+		Expect(err).ToNot(HaveOccurred())
+		catalogConfigMap.Data[registry.ConfigMapCSVName] = string(csvsRaw)
+	}
+
+	createdConfigMap, err := c.KubernetesInterface().CoreV1().ConfigMaps(namespace).Create(context.TODO(), catalogConfigMap, metav1.CreateOptions{})
+	if err != nil && !apierrors.IsAlreadyExists(err) {
+		Expect(err).ToNot(HaveOccurred())
+	}
+	return createdConfigMap, buildConfigMapCleanupFunc(c, namespace, createdConfigMap)
+}
+
+func createV1CRDConfigMapForCatalogData(t GinkgoTInterface, c operatorclient.ClientInterface, name, namespace string, manifests []registry.PackageManifest, crds []apiextensionsv1.CustomResourceDefinition, csvs []v1alpha1.ClusterServiceVersion) (*corev1.ConfigMap, cleanupFunc) {
 	// Create a config map containing the PackageManifests and CSVs
 	configMapName := fmt.Sprintf("%s-configmap", name)
 	catalogConfigMap := &corev1.ConfigMap{
@@ -530,7 +598,7 @@ func createConfigMapForCatalogData(t *testing.T, c operatorclient.ClientInterfac
 	if crds != nil {
 		crdStrings := []string{}
 		for _, crd := range crds {
-			crdStrings = append(crdStrings, serializeCRD(t, crd))
+			crdStrings = append(crdStrings, serializeV1CRD(t, &crd))
 		}
 		var err error
 		crdsRaw, err = yaml.Marshal(crdStrings)
@@ -545,23 +613,22 @@ func createConfigMapForCatalogData(t *testing.T, c operatorclient.ClientInterfac
 		catalogConfigMap.Data[registry.ConfigMapCSVName] = string(csvsRaw)
 	}
 
-	createdConfigMap, err := c.KubernetesInterface().CoreV1().ConfigMaps(namespace).Create(catalogConfigMap)
-	if err != nil && !errors.IsAlreadyExists(err) {
+	createdConfigMap, err := c.KubernetesInterface().CoreV1().ConfigMaps(namespace).Create(context.TODO(), catalogConfigMap, metav1.CreateOptions{})
+	if err != nil && !apierrors.IsAlreadyExists(err) {
 		require.NoError(t, err)
 	}
-	return createdConfigMap, buildConfigMapCleanupFunc(t, c, namespace, createdConfigMap)
+	return createdConfigMap, buildConfigMapCleanupFunc(c, namespace, createdConfigMap)
 }
 
-func serializeCRD(t *testing.T, crd apiextensions.CustomResourceDefinition) string {
+func serializeCRD(crd apiextensions.CustomResourceDefinition) string {
 	scheme := runtime.NewScheme()
-	extScheme.AddToScheme(scheme)
-	k8sscheme.AddToScheme(scheme)
-	err := v1beta1.AddToScheme(scheme)
-	require.NoError(t, err)
+
+	Expect(extScheme.AddToScheme(scheme)).Should(Succeed())
+	Expect(k8sscheme.AddToScheme(scheme)).Should(Succeed())
+	Expect(v1beta1.AddToScheme(scheme)).Should(Succeed())
 
 	out := &v1beta1.CustomResourceDefinition{}
-	err = scheme.Convert(&crd, out, nil)
-	require.NoError(t, err)
+	Expect(scheme.Convert(&crd, out, nil)).To(Succeed())
 	out.TypeMeta = metav1.TypeMeta{
 		Kind:       "CustomResourceDefinition",
 		APIVersion: "apiextensions.k8s.io/v1beta1",
@@ -572,17 +639,21 @@ func serializeCRD(t *testing.T, crd apiextensions.CustomResourceDefinition) stri
 
 	// create an object manifest
 	var manifest bytes.Buffer
-	err = serializer.Encode(out, &manifest)
-	require.NoError(t, err)
+	Expect(serializer.Encode(out, &manifest)).To(Succeed())
 	return manifest.String()
 }
 
-func serializeObject(obj interface{}) string {
-	bytes, err := json.Marshal(obj)
-	if err != nil {
-		return ""
-	}
-	return string(bytes)
+func serializeV1CRD(t GinkgoTInterface, crd *apiextensionsv1.CustomResourceDefinition) string {
+	scheme := runtime.NewScheme()
+	require.NoError(t, apiextensionsv1.AddToScheme(scheme))
+
+	// set up object serializer
+	serializer := k8sjson.NewYAMLSerializer(k8sjson.DefaultMetaFactory, scheme, scheme)
+
+	// create an object manifest
+	var manifest bytes.Buffer
+	require.NoError(t, serializer.Encode(crd, &manifest))
+	return manifest.String()
 }
 
 func createCR(c operatorclient.ClientInterface, item *unstructured.Unstructured, apiGroup, version, namespace, resourceKind, resourceName string) (cleanupFunc, error) {
@@ -617,7 +688,7 @@ func Local(client operatorclient.ClientInterface) (bool, error) {
 
 	groups, err := client.KubernetesInterface().Discovery().ServerResourcesForGroupVersion(gv)
 	if err != nil {
-		if errors.IsNotFound(err) {
+		if apierrors.IsNotFound(err) {
 			return true, nil
 		}
 		return true, fmt.Errorf("checking if cluster is local: checking server groups: %s", err)
@@ -632,24 +703,182 @@ func Local(client operatorclient.ClientInterface) (bool, error) {
 	return true, nil
 }
 
-func newSSAClient(t *testing.T, config *rest.Config) *controllerclient.ServerSideApplier {
-	scheme := runtime.NewScheme()
-	localSchemeBuilder := runtime.NewSchemeBuilder(
-		apiextensionsv1.AddToScheme,
-		kscheme.AddToScheme,
-		v1alpha1.AddToScheme,
-		operatorsv1.AddToScheme,
+// predicateFunc is a predicate for watch events.
+type predicateFunc func(event watch.Event) (met bool)
+
+// awaitPredicates waits for all predicates to be met by events of a watch in the order given.
+func awaitPredicates(ctx context.Context, w watch.Interface, fns ...predicateFunc) {
+	if len(fns) < 1 {
+		panic("no predicates given to await")
+	}
+
+	i := 0
+	for i < len(fns) {
+		select {
+		case <-ctx.Done():
+			Expect(ctx.Err()).ToNot(HaveOccurred())
+			return
+		case event, ok := <-w.ResultChan():
+			if !ok {
+				return
+			}
+
+			if fns[i](event) {
+				i++
+			}
+		}
+	}
+}
+
+// filteredPredicate filters events to the given predicate by event type to the given types.
+// When no event types are given as arguments, all event types are passed through.
+func filteredPredicate(fn predicateFunc, eventTypes ...watch.EventType) predicateFunc {
+	return func(event watch.Event) bool {
+		valid := true
+		for _, eventType := range eventTypes {
+			if valid = eventType == event.Type; valid {
+				break
+			}
+		}
+
+		if !valid {
+			return false
+		}
+
+		return fn(event)
+	}
+}
+
+func deploymentPredicate(fn func(*appsv1.Deployment) bool) predicateFunc {
+	return func(event watch.Event) bool {
+		deployment, ok := event.Object.(*appsv1.Deployment)
+		Expect(ok).To(BeTrue(), "unexpected event object type %T in deployment", event.Object)
+
+		return fn(deployment)
+	}
+}
+
+var deploymentAvailable = filteredPredicate(deploymentPredicate(func(deployment *appsv1.Deployment) bool {
+	for _, condition := range deployment.Status.Conditions {
+		if condition.Type == appsv1.DeploymentAvailable && condition.Status == corev1.ConditionTrue {
+			return true
+		}
+	}
+
+	return false
+}), watch.Added, watch.Modified)
+
+func deploymentReplicas(replicas int32) predicateFunc {
+	return filteredPredicate(deploymentPredicate(func(deployment *appsv1.Deployment) bool {
+		return deployment.Status.Replicas == replicas
+	}), watch.Added, watch.Modified)
+}
+
+const (
+	cvoNamespace      = "openshift-cluster-version"
+	cvoDeploymentName = "cluster-version-operator"
+)
+
+func toggleCVO() {
+	c := ctx.Ctx().KubeClient().KubernetesInterface().AppsV1().Deployments(cvoNamespace)
+	clientCtx := context.Background()
+	scale, err := c.GetScale(clientCtx, cvoDeploymentName, metav1.GetOptions{})
+	if err != nil {
+		if apierrors.IsNotFound(err) {
+			// CVO is not enabled
+			return
+		}
+
+		Expect(err).ToNot(HaveOccurred())
+	}
+
+	if scale.Spec.Replicas > 0 {
+		scale.Spec.Replicas = 0
+	} else {
+		scale.Spec.Replicas = 1
+	}
+
+	Eventually(func() error {
+		_, err := c.UpdateScale(clientCtx, cvoDeploymentName, scale, metav1.UpdateOptions{})
+		return err
+	}).Should(Succeed())
+}
+
+// togglev2alpha1 toggles the v2alpha1 feature gate on or off.
+func togglev2alpha1() {
+	// Set the feature flag on OLM's deployment
+	c := ctx.Ctx().KubeClient()
+	deployment, err := getOperatorDeployment(c, operatorNamespace, labels.Set{"app": "olm-operator"})
+	Expect(err).ToNot(HaveOccurred())
+	toggleFeatureGates(deployment, feature.OperatorLifecycleManagerV2)
+}
+
+// toggleFeatureGates toggles the given feature gates on or off based on their current setting in the deployment.
+func toggleFeatureGates(deployment *appsv1.Deployment, toToggle ...featuregate.Feature) {
+	var (
+		c              = ctx.Ctx().KubeClient().KubernetesInterface().AppsV1().Deployments(deployment.GetNamespace())
+		containers     = deployment.Spec.Template.Spec.Containers
+		containerIndex = -1
+		argIndex       = -1
+		prefix         = "--feature-gates="
+		gateVals       string
 	)
 
-	err := localSchemeBuilder.AddToScheme(scheme)
-	require.NoError(t, err)
+	// Find the container and argument indices for the feature gate option
+	for i, container := range containers {
+		if container.Name != "olm-operator" {
+			continue
+		}
+		containerIndex = i
 
-	ssaClient, err := controllerclient.NewForConfig(config, scheme, "test.olm.registry")
-	require.NoError(t, err)
-	return ssaClient
+		for j, arg := range container.Args {
+			if gateVals = strings.TrimPrefix(arg, prefix); arg == gateVals {
+				continue
+			}
+			argIndex = j
+
+			break
+		}
+
+		break
+	}
+	// This should never happen since Deployments must have at least one container
+	Expect(containerIndex).ToNot(BeNumerically("<", 0), "deployment %s has no containers", deployment.GetName())
+
+	gate := feature.Gate.DeepCopy()
+	if argIndex >= 0 {
+		// Collect existing gate values
+		Expect(gate.Set(gateVals)).To(Succeed())
+	}
+
+	// Toggle gates
+	toggled := map[string]bool{}
+	for _, feature := range toToggle {
+		toggled[string(feature)] = !gate.Enabled(feature)
+	}
+	Expect(gate.SetFromMap(toggled)).To(Succeed())
+
+	gateArg := fmt.Sprintf("%s%s", prefix, gate)
+	if argIndex >= 0 {
+		// Overwrite existing gate options
+		containers[containerIndex].Args[argIndex] = gateArg
+	} else {
+		// No existing gate options, add one
+		containers[containerIndex].Args = append(containers[containerIndex].Args, gateArg)
+	}
+
+	clientCtx := context.Background()
+	w, err := c.Watch(clientCtx, metav1.ListOptions{})
+	Expect(err).ToNot(HaveOccurred())
+
+	timeout := 1 * time.Minute
+	Eventually(func() error {
+		_, err := c.Update(clientCtx, deployment, metav1.UpdateOptions{})
+		return err
+	}, timeout).Should(Succeed())
+
+	deadline, cancel := context.WithTimeout(context.Background(), timeout)
+	defer cancel()
+
+	awaitPredicates(deadline, w, deploymentReplicas(2), deploymentAvailable, deploymentReplicas(1))
 }
-
-func Apply(ssaClient *controllerclient.ServerSideApplier, obj controllerclient.Object, changeFunc interface{}) func() error {
-	return ssaClient.Apply(context.Background(), obj, changeFunc)
-}
-
